@@ -62,7 +62,14 @@ import io.opentelemetry.context.Context;
 import io.opentelemetry.context.Scope;
 import io.opentelemetry.context.propagation.TextMapSetter;
 import io.opentelemetry.sdk.testing.junit4.OpenTelemetryRule;
-import io.opentelemetry.semconv.trace.attributes.SemanticAttributes;
+import io.opentelemetry.sdk.trace.data.SpanData;
+import io.opentelemetry.semconv.ClientAttributes;
+import io.opentelemetry.semconv.ErrorAttributes;
+import io.opentelemetry.semconv.HttpAttributes;
+import io.opentelemetry.semconv.NetworkAttributes;
+import io.opentelemetry.semconv.ServerAttributes;
+import io.opentelemetry.semconv.UrlAttributes;
+import io.opentelemetry.semconv.UserAgentAttributes;
 
 import org.junit.After;
 import org.junit.BeforeClass;
@@ -75,10 +82,12 @@ import static org.apache.cxf.systest.jaxrs.tracing.opentelemetry.HasAttribute.ha
 import static org.apache.cxf.systest.jaxrs.tracing.opentelemetry.HasSpan.hasSpan;
 import static org.apache.cxf.systest.jaxrs.tracing.opentelemetry.IsLogContaining.hasItem;
 import static org.awaitility.Awaitility.await;
+import static org.hamcrest.CoreMatchers.containsString;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.equalTo;
 import static org.hamcrest.Matchers.notNullValue;
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
 
 public class OpenTelemetryTracingTest extends AbstractClientServerTestBase {
@@ -93,7 +102,7 @@ public class OpenTelemetryTracingTest extends AbstractClientServerTestBase {
     public ExpectedException expectedException = ExpectedException.none();
 
     @BeforeClass
-    public static void startServers() throws Exception {
+    public static void startServers() {
         AbstractResourceInfo.clearAllMaps();
         // keep out of process due to stack traces testing failures
         assertTrue("server did not launch correctly", launchServer(OpenTelemetryServer.class, true));
@@ -124,11 +133,23 @@ public class OpenTelemetryTracingTest extends AbstractClientServerTestBase {
 
         assertThat(otelRule.getSpans().size(), equalTo(2));
         assertThat(otelRule.getSpans().get(0).getName(), equalTo("Get Books"));
-        assertThat(otelRule.getSpans().get(1).getName(), equalTo("GET /bookstore/books"));
-        assertThat(otelRule.getSpans().get(1).getAttributes(),
-                   hasAttribute(SemanticAttributes.HTTP_STATUS_CODE, 200L));
-        assertThat(otelRule.getSpans().get(1).getInstrumentationScopeInfo().getName(),
-            equalTo("jaxrs-server-test"));
+        SpanData serverSpan = otelRule.getSpans().get(1);
+        assertThat(serverSpan.getName(), equalTo("GET /bookstore/books"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(HttpAttributes.HTTP_RESPONSE_STATUS_CODE, 200L));
+        assertThat(serverSpan.getAttributes(), hasAttribute(HttpAttributes.HTTP_REQUEST_METHOD, "GET"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(UrlAttributes.URL_PATH, "/bookstore/books"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(ServerAttributes.SERVER_ADDRESS, "localhost"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(ServerAttributes.SERVER_PORT, Long.valueOf(PORT)));
+        assertThat(serverSpan.getAttributes(), hasAttribute(NetworkAttributes.NETWORK_PEER_ADDRESS, "127.0.0.1"));
+        assertNotNull(serverSpan.getAttributes().get(NetworkAttributes.NETWORK_PEER_PORT));
+        assertThat(serverSpan.getAttributes(), hasAttribute(ClientAttributes.CLIENT_ADDRESS, "127.0.0.1"));
+        assertNotNull(serverSpan.getAttributes().get(ClientAttributes.CLIENT_PORT));
+        assertThat(serverSpan.getAttributes(), hasAttribute(UrlAttributes.URL_SCHEME, "http"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(NetworkAttributes.NETWORK_PROTOCOL_VERSION, "1.1"));
+        String userAgent = serverSpan.getAttributes().get(UserAgentAttributes.USER_AGENT_ORIGINAL);
+        assertNotNull(userAgent);
+        assertThat(userAgent, containsString("Apache-CXF/"));
+        assertThat(serverSpan.getInstrumentationScopeInfo().getName(), equalTo("jaxrs-server-test"));
     }
 
     @Test
@@ -187,6 +208,45 @@ public class OpenTelemetryTracingTest extends AbstractClientServerTestBase {
         assertThat(otelRule.getSpans().get(2).getInstrumentationScopeInfo().getName(), equalTo("jaxrs-client-test"));
     }
 
+    @Test
+    public void spanShouldHasRequiredAttributes() {
+        final Response r = createWebClient("/bookstore/books",
+                new OpenTelemetryClientProvider(otelRule.getOpenTelemetry(), "jaxrs-client-test"))
+                .get();
+        assertEquals(Status.OK.getStatusCode(), r.getStatus());
+
+        assertThat(otelRule.getSpans().toString(), otelRule.getSpans().size(), equalTo(3));
+
+        SpanData clientSpan = otelRule.getSpans().get(2);
+        assertThat(clientSpan.getName(), equalTo("GET http://localhost:" + PORT + "/bookstore/books"));
+        assertThat(clientSpan.getAttributes(), hasAttribute(HttpAttributes.HTTP_RESPONSE_STATUS_CODE, 200L));
+        assertThat(clientSpan.getAttributes(), hasAttribute(HttpAttributes.HTTP_REQUEST_METHOD, "GET"));
+        assertThat(clientSpan.getAttributes(), hasAttribute(UrlAttributes.URL_FULL,
+                "http://localhost:" + PORT + "/bookstore/books"));
+        assertThat(clientSpan.getAttributes(), hasAttribute(ServerAttributes.SERVER_ADDRESS, "localhost"));
+        assertThat(clientSpan.getAttributes(), hasAttribute(ServerAttributes.SERVER_PORT, Long.valueOf(PORT)));
+        assertThat(clientSpan.getAttributes(), hasAttribute(NetworkAttributes.NETWORK_PEER_ADDRESS, "localhost"));
+        assertThat(clientSpan.getAttributes(), hasAttribute(NetworkAttributes.NETWORK_PEER_PORT, Long.valueOf(PORT)));
+
+        SpanData serverSpan = otelRule.getSpans().get(1);
+        assertThat(serverSpan.getName(), equalTo("GET /bookstore/books"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(HttpAttributes.HTTP_RESPONSE_STATUS_CODE, 200L));
+        assertThat(serverSpan.getAttributes(), hasAttribute(HttpAttributes.HTTP_REQUEST_METHOD, "GET"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(UrlAttributes.URL_PATH, "/bookstore/books"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(ServerAttributes.SERVER_ADDRESS, "localhost"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(ServerAttributes.SERVER_PORT, Long.valueOf(PORT)));
+        assertThat(serverSpan.getAttributes(), hasAttribute(NetworkAttributes.NETWORK_PEER_ADDRESS, "127.0.0.1"));
+        assertNotNull(serverSpan.getAttributes().get(NetworkAttributes.NETWORK_PEER_PORT));
+        assertThat(serverSpan.getAttributes(), hasAttribute(ClientAttributes.CLIENT_ADDRESS, "127.0.0.1"));
+        assertNotNull(serverSpan.getAttributes().get(ClientAttributes.CLIENT_PORT));
+        assertThat(serverSpan.getAttributes(), hasAttribute(UrlAttributes.URL_SCHEME, "http"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(NetworkAttributes.NETWORK_PROTOCOL_VERSION, "1.1"));
+        String userAgent = serverSpan.getAttributes().get(UserAgentAttributes.USER_AGENT_ORIGINAL);
+        assertNotNull(userAgent);
+        assertThat(userAgent, containsString("Apache-CXF/"));
+        assertThat(serverSpan.getInstrumentationScopeInfo().getName(), equalTo("jaxrs-server-test"));
+    }
+
     @Test
     public void testThatNewInnerSpanIsCreatedUsingAsyncInvocation() throws InterruptedException {
         final Context parentContext = fromRandom();
@@ -254,7 +314,7 @@ public class OpenTelemetryTracingTest extends AbstractClientServerTestBase {
         final WebClient client = createWebClient("/bookstore/books",
             new OpenTelemetryClientProvider(otelRule.getOpenTelemetry(), "jaxrs-client-test"));
 
-        // The intention is to make a calls one after another, not in parallel, to ensure the
+        // The intention is to make multiple calls one after another, not in parallel, to ensure the
         // thread have trace contexts cleared out.
         IntStream.range(0, 4).mapToObj(index -> client.async().get()).map(this::get)
             .forEach(r -> assertEquals(Status.OK.getStatusCode(), r.getStatus()));
@@ -264,8 +324,7 @@ public class OpenTelemetryTracingTest extends AbstractClientServerTestBase {
         IntStream.range(0, 4).map(index -> index * 3).forEach(index -> {
             assertThat(otelRule.getSpans().get(index).getName(), equalTo("Get Books"));
             assertThat(otelRule.getSpans().get(index + 1).getName(), equalTo("GET /bookstore/books"));
-            assertThat(otelRule.getSpans().get(index + 2).getName(),
-                       equalTo("GET " + client.getCurrentURI()));
+            assertThat(otelRule.getSpans().get(index + 2).getName(), equalTo("GET " + client.getCurrentURI()));
         });
     }
 
@@ -274,7 +333,7 @@ public class OpenTelemetryTracingTest extends AbstractClientServerTestBase {
         final WebClient client = createWebClient("/bookstore/books",
             new OpenTelemetryClientProvider(otelRule.getOpenTelemetry(), "jaxrs-client-test"));
 
-        // The intention is to make a calls one after another, not in parallel, to ensure the
+        // The intention is to make multiple calls one after another, not in parallel, to ensure the
         // thread have trace contexts cleared out.
         IntStream.range(0, 4).mapToObj(index -> client.get())
             .forEach(r -> assertEquals(Status.OK.getStatusCode(), r.getStatus()));
@@ -284,8 +343,7 @@ public class OpenTelemetryTracingTest extends AbstractClientServerTestBase {
         IntStream.range(0, 4).map(index -> index * 3).forEach(index -> {
             assertThat(otelRule.getSpans().get(index).getName(), equalTo("Get Books"));
             assertThat(otelRule.getSpans().get(index + 1).getName(), equalTo("GET /bookstore/books"));
-            assertThat(otelRule.getSpans().get(index + 2).getName(),
-                       equalTo("GET " + client.getCurrentURI()));
+            assertThat(otelRule.getSpans().get(index + 2).getName(), equalTo("GET " + client.getCurrentURI()));
         });
     }
 
@@ -395,9 +453,37 @@ public class OpenTelemetryTracingTest extends AbstractClientServerTestBase {
         assertEquals(Status.INTERNAL_SERVER_ERROR.getStatusCode(), r.getStatus());
 
         assertThat(otelRule.getSpans().toString(), otelRule.getSpans().size(), equalTo(1));
-        assertThat(otelRule.getSpans().get(0).getName(), equalTo("GET /bookstore/books/exception"));
-        assertThat(otelRule.getSpans().get(0).getAttributes(),
-                   hasAttribute(SemanticAttributes.HTTP_STATUS_CODE, 500L));
+        SpanData serverSpan = otelRule.getSpans().get(0);
+
+        assertThat(serverSpan.getName(), equalTo("GET /bookstore/books/exception"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(HttpAttributes.HTTP_RESPONSE_STATUS_CODE, 500L));
+        assertThat(serverSpan.getAttributes(), hasAttribute(HttpAttributes.HTTP_REQUEST_METHOD, "GET"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(UrlAttributes.URL_PATH, "/bookstore/books/exception"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(ServerAttributes.SERVER_ADDRESS, "localhost"));
+        assertThat(serverSpan.getAttributes(), hasAttribute(ServerAttributes.SERVER_PORT, Long.valueOf(PORT)));
+        assertThat(serverSpan.getAttributes(), hasAttribute(ErrorAttributes.ERROR_TYPE,  String.valueOf(500)));
+    }
+
+    @Test
+    public void testClientSpanAttributesOnException() {
+        final Response r = createWebClient("/bookstore/books/exception",
+                new OpenTelemetryClientProvider(otelRule.getOpenTelemetry(), "jaxrs-client-test"))
+                .get();
+
+        assertEquals(Status.INTERNAL_SERVER_ERROR.getStatusCode(), r.getStatus());
+
+        assertThat(otelRule.getSpans().toString(), otelRule.getSpans().size(), equalTo(2));
+
+        SpanData clientSpan = otelRule.getSpans().get(1);
+        assertThat(clientSpan.getName(), equalTo("GET http://localhost:" + PORT + "/bookstore/books/exception"));
+        assertThat(clientSpan.getAttributes(), hasAttribute(HttpAttributes.HTTP_REQUEST_METHOD, "GET"));
+        assertThat(clientSpan.getAttributes(), hasAttribute(UrlAttributes.URL_FULL,
+                "http://localhost:" + PORT + "/bookstore/books/exception"));
+        assertThat(clientSpan.getAttributes(), hasAttribute(ServerAttributes.SERVER_ADDRESS, "localhost"));
+        assertThat(clientSpan.getAttributes(), hasAttribute(ServerAttributes.SERVER_PORT, Long.valueOf(PORT)));
+        assertThat(clientSpan.getAttributes(), hasAttribute(NetworkAttributes.NETWORK_PEER_ADDRESS, "localhost"));
+        assertThat(clientSpan.getAttributes(), hasAttribute(NetworkAttributes.NETWORK_PEER_PORT, Long.valueOf(PORT)));
+        assertThat(clientSpan.getAttributes(), hasAttribute(ErrorAttributes.ERROR_TYPE,  String.valueOf(500)));
     }
 
     @Test
@@ -408,7 +494,7 @@ public class OpenTelemetryTracingTest extends AbstractClientServerTestBase {
         assertThat(otelRule.getSpans().toString(), otelRule.getSpans().size(), equalTo(1));
         assertThat(otelRule.getSpans().get(0).getName(), equalTo("GET /bookstore/books/error"));
         assertThat(otelRule.getSpans().get(0).getAttributes(),
-                   hasAttribute(SemanticAttributes.HTTP_STATUS_CODE, 503L));
+                   hasAttribute(HttpAttributes.HTTP_RESPONSE_STATUS_CODE, 503L));
     }
 
     @Test
@@ -419,7 +505,7 @@ public class OpenTelemetryTracingTest extends AbstractClientServerTestBase {
         assertThat(otelRule.getSpans().toString(), otelRule.getSpans().size(), equalTo(1));
         assertThat(otelRule.getSpans().get(0).getName(), equalTo("GET /bookstore/books/mapper"));
         assertThat(otelRule.getSpans().get(0).getAttributes(),
-                   hasAttribute(SemanticAttributes.HTTP_STATUS_CODE, 404L));
+                   hasAttribute(HttpAttributes.HTTP_RESPONSE_STATUS_CODE, 404L));
     }
 
     private WebClient withTrace(final WebClient client) {
